#!/usr/bin/env python
# -*- coding:utf-8 -*-
# Author: wxnacy(wxnacy@gmail.com)
"""

"""

from wpy.base import BaseEnum

class BasetypeEnum(BaseEnum):
    STR = str
    INT = int
    FLOAT = float
    LIST = list
    DICT = dict
